"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.renderToWebStream = void 0;
const createReadableWrapper_1 = require("./createReadableWrapper");
const misc_1 = require("./misc");
async function renderToWebStream(element, disable, options) {
    var _a;
    (0, misc_1.debugFlow)('creating Web Stream Pipe');
    let didError = false;
    let firstErr = null;
    let reactBug = null;
    const onError = (err) => {
        didError = true;
        firstErr = firstErr || err;
        (0, misc_1.afterReactBugCatch)(() => {
            var _a;
            // Is not a React internal error (i.e. a React bug)
            if (err !== reactBug) {
                (_a = options.onBoundaryError) === null || _a === void 0 ? void 0 : _a.call(options, err);
            }
        });
    };
    const renderToReadableStream = (_a = options.renderToReadableStream) !== null && _a !== void 0 ? _a : 
    // We directly use import() because it needs to be bundled for Cloudflare Workers
    (await import('react-dom/server.browser')).renderToReadableStream;
    (0, misc_1.assertReactImport)(renderToReadableStream, 'renderToReadableStream');
    const readableOriginal = await renderToReadableStream(element, { onError });
    const { allReady } = readableOriginal;
    let promiseResolved = false;
    // Upon React internal errors (i.e. React bugs), React rejects `allReady`.
    // React doesn't reject `allReady` upon boundary errors.
    allReady.catch((err) => {
        (0, misc_1.debugFlow)('react bug');
        didError = true;
        firstErr = firstErr || err;
        reactBug = err;
        // Only log if it wasn't used as rejection for `await renderToStream()`
        if (reactBug !== firstErr || promiseResolved) {
            console.error(reactBug);
        }
    });
    if (didError)
        throw firstErr;
    if (disable)
        await allReady;
    if (didError)
        throw firstErr;
    const { readableForUser, streamEnd, injectToStream } = (0, createReadableWrapper_1.createReadableWrapper)(readableOriginal);
    promiseResolved = true;
    return {
        readable: readableForUser,
        pipe: null,
        streamEnd: (0, misc_1.wrapStreamEnd)(streamEnd, didError),
        injectToStream
    };
}
exports.renderToWebStream = renderToWebStream;
