export { createPipeWrapper };
import { createDebugger } from '../utils';
import { createBuffer } from './createBuffer';
const debug = createDebugger('react-streaming:createPipeWrapper');
import { Writable } from 'stream';
async function createPipeWrapper(pipeFromReact, { onReactBug }) {
    const { pipeForUser, streamEnd } = createPipeForUser();
    const streamOperations = {
        operations: null
    };
    const { injectToStream, onBeforeWrite, onBeforeEnd } = createBuffer(streamOperations);
    return { pipeForUser, streamEnd, injectToStream };
    function createPipeForUser() {
        debug('createPipeForUser()');
        let onEnded;
        const streamEnd = new Promise((r) => {
            onEnded = () => r();
        });
        const pipeForUser = (writableFromUser) => {
            const writableForReact = new Writable({
                write(chunk, encoding, callback) {
                    debug('write');
                    onBeforeWrite(chunk);
                    if (!writableFromUser.destroyed) {
                        writableFromUser.write(chunk, encoding, callback);
                    }
                    else {
                        // Destroying twice is fine: https://github.com/brillout/react-streaming/pull/21#issuecomment-1554517163
                        writableForReact.destroy();
                    }
                },
                final(callback) {
                    debug('final');
                    onBeforeEnd();
                    writableFromUser.end();
                    onEnded();
                    callback();
                },
                destroy(err) {
                    debug(`destroy (\`!!err === ${!!err}\`)`);
                    // Upon React internal errors (i.e. React bugs), React destroys the stream.
                    if (err)
                        onReactBug(err);
                    writableFromUser.destroy(err !== null && err !== void 0 ? err : undefined);
                    onEnded();
                }
            });
            const flush = () => {
                if (typeof writableFromUser.flush === 'function') {
                    ;
                    writableFromUser.flush();
                    debug('stream flushed (Node.js Writable)');
                }
            };
            streamOperations.operations = {
                flush,
                writeChunk(chunk) {
                    writableFromUser.write(chunk);
                }
            };
            writableForReact.flush = flush;
            pipeFromReact(writableForReact);
        };
        return { pipeForUser, streamEnd };
    }
}
